@@ -1,6 +1,8 @@
 #[cfg(feature = "std")]
 pub use stdio::*;
 
+use either::*;
+
 const BUF_SIZE: usize = 128;
 
 pub trait Read {
@@ -55,37 +57,54 @@ where
     }
 }
 
-pub fn copy<R, W>(read: R, write: W) -> Result<u64, R::Error>
+pub fn copy<R, W>(read: R, write: W) -> Result<u64, Either<R::Error, W::Error>>
 where
     R: Read,
     W: Write,
-    W::Error: Into<R::Error>,
 {
     copy_len(read, write, u64::MAX)
 }
 
-pub fn copy_len<R, W>(mut read: R, mut write: W, mut len: u64) -> Result<u64, R::Error>
+pub fn copy_len<R, W>(read: R, write: W, len: u64) -> Result<u64, Either<R::Error, W::Error>>
+where
+    R: Read,
+    W: Write,
+{
+    copy_len_with_progress(read, write, len, |_, _| {})
+}
+
+pub fn copy_len_with_progress<R, W>(
+    mut read: R,
+    mut write: W,
+    mut len: u64,
+    progress: impl Fn(u64, u64),
+) -> Result<u64, Either<R::Error, W::Error>>
 where
     R: Read,
     W: Write,
-    W::Error: Into<R::Error>,
 {
     let mut buf = [0_u8; BUF_SIZE];
 
     let mut copied = 0;
 
     while len > 0 {
-        let size_read = read.do_read(&mut buf)?;
+        progress(copied, len);
+
+        let size_read = read.do_read(&mut buf).map_err(Either::Left)?;
         if size_read == 0 {
             break;
         }
 
-        write.do_write_all(&buf[0..size_read]).map_err(Into::into)?;
+        write
+            .do_write_all(&buf[0..size_read])
+            .map_err(Either::Right)?;
 
         copied += size_read as u64;
         len -= size_read as u64;
     }
 
+    progress(copied, len);
+
     Ok(copied)
 }
 
